@@ -444,7 +444,7 @@ action_about_nautilus_callback (GtkAction *action,
 					     "for GNOME that makes it "
 					     "easy to manage your files "
 					     "and the rest of your system."),
-			       "copyright", _("Copyright \xC2\xA9 1999-2006 "
+			       "copyright", _("Copyright \xC2\xA9 1999-2007 "
 					      "The Nautilus authors"),
 			       "license", license_trans,
 			       "wrap-license", TRUE,
@@ -456,6 +456,8 @@ action_about_nautilus_callback (GtkAction *action,
 				 */
 			      "translator-credits", _("translator-credits"),
 			      "logo-icon-name", "nautilus",
+			      "website", "http://www.gnome.org/projects/nautilus",
+			      "website-label", "Nautilus Website",			     
 			      NULL);
 
 	g_free (license_trans);
@@ -673,7 +675,7 @@ static const GtkActionEntry main_entries[] = {
     N_("Open your personal trash folder"),                                  /* tooltip */ 
     G_CALLBACK (action_go_to_trash_callback) },
   { "Go to Burn CD", "nautilus-cd-burner",                        /* name, stock id */
-    N_("CD/_DVD"), NULL,           /* label, accelerator */
+    N_("CD/_DVD Creator"), NULL,           /* label, accelerator */
     N_("Open a folder into which you can drag files to burn to a CD or DVD"),                                  /* tooltip */ 
     G_CALLBACK (action_go_to_burn_cd_callback) },
 };
